"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MapsToTransformer = void 0;
const graphql_transformer_core_1 = require("@aws-amplify/graphql-transformer-core");
const graphql_transformer_interfaces_1 = require("@aws-amplify/graphql-transformer-interfaces");
const graphql_1 = require("graphql");
const field_mapping_lambda_1 = require("./field-mapping-lambda");
const field_mapping_resolvers_1 = require("./field-mapping-resolvers");
const directiveName = 'mapsTo';
const directiveDefinition = `
  directive @${directiveName}(name: String!) on OBJECT
`;
class MapsToTransformer extends graphql_transformer_core_1.TransformerPluginBase {
    constructor() {
        super(`amplify-maps-to-transformer`, directiveDefinition, graphql_transformer_interfaces_1.TransformerPluginType.GENERIC);
        this.object = (definition, directive, ctx) => {
            updateTypeMapping(definition, directive, ctx.inputDocument, ctx.resourceHelper.setModelNameMapping);
        };
        this.preMutateSchema = (context) => {
            var _a, _b;
            (_b = (_a = context.inputDocument) === null || _a === void 0 ? void 0 : _a.definitions) === null || _b === void 0 ? void 0 : _b.forEach((def) => {
                var _a;
                if (def.kind === 'ObjectTypeDefinition' || def.kind === 'ObjectTypeExtension') {
                    (_a = def === null || def === void 0 ? void 0 : def.directives) === null || _a === void 0 ? void 0 : _a.forEach((dir) => {
                        if (dir.name.value === directiveName) {
                            updateTypeMapping(def, dir, context.inputDocument, context.schemaHelper.setTypeMapping);
                        }
                    });
                }
            });
        };
        this.after = (context) => {
            context.resourceHelper.getModelFieldMapKeys().forEach((modelName) => {
                const modelFieldMap = context.resourceHelper.getModelFieldMap(modelName);
                if (!modelFieldMap.getMappedFields().length) {
                    return;
                }
                const lambdaDataSource = (0, field_mapping_lambda_1.createMappingLambda)(context.api.host, context.stackManager);
                modelFieldMap.getResolverReferences().forEach(({ typeName, fieldName, isList }) => {
                    const resolver = context.resolvers.getResolver(typeName, fieldName);
                    if (!resolver) {
                        return;
                    }
                    if (typeName === 'Mutation') {
                        (0, field_mapping_resolvers_1.attachInputMappingSlot)({
                            resolver,
                            resolverTypeName: typeName,
                            resolverFieldName: fieldName,
                            fieldMap: modelFieldMap.getMappedFields(),
                        });
                        (0, field_mapping_resolvers_1.attachFilterAndConditionInputMappingSlot)({
                            slotName: 'preUpdate',
                            resolver,
                            resolverTypeName: typeName,
                            resolverFieldName: fieldName,
                            fieldMap: modelFieldMap.getMappedFields(),
                            dataSource: lambdaDataSource,
                        });
                        (0, field_mapping_resolvers_1.attachResponseMappingSlot)({
                            slotName: 'postUpdate',
                            resolver,
                            resolverTypeName: typeName,
                            resolverFieldName: fieldName,
                            fieldMap: modelFieldMap.getMappedFields(),
                            isList: false,
                        });
                    }
                    else {
                        (0, field_mapping_resolvers_1.attachFilterAndConditionInputMappingSlot)({
                            slotName: 'preDataLoad',
                            resolver,
                            resolverTypeName: typeName,
                            resolverFieldName: fieldName,
                            fieldMap: modelFieldMap.getMappedFields(),
                            dataSource: lambdaDataSource,
                        });
                        (0, field_mapping_resolvers_1.attachResponseMappingSlot)({
                            slotName: 'postDataLoad',
                            resolver,
                            resolverTypeName: typeName,
                            resolverFieldName: fieldName,
                            fieldMap: modelFieldMap.getMappedFields(),
                            isList,
                        });
                    }
                });
            });
        };
    }
}
exports.MapsToTransformer = MapsToTransformer;
const updateTypeMapping = (definition, directive, inputDocument, updateFunction) => {
    var _a, _b;
    const modelName = definition.name.value;
    const prevNameNode = (_a = directive.arguments) === null || _a === void 0 ? void 0 : _a.find((arg) => arg.name.value === 'name');
    const hasModelDirective = !!((_b = definition.directives) === null || _b === void 0 ? void 0 : _b.find((directive) => directive.name.value === 'model'));
    if (!hasModelDirective) {
        throw new graphql_transformer_core_1.InvalidDirectiveError(`@mapsTo can only be used on an @model type`);
    }
    if (!prevNameNode) {
        throw new graphql_transformer_core_1.InvalidDirectiveError(`name is required in @${directiveName} directive`);
    }
    if (prevNameNode.value.kind !== 'StringValue') {
        throw new graphql_transformer_core_1.InvalidDirectiveError(`A single string must be provided for "name" in @${directiveName} directive`);
    }
    const originalName = prevNameNode.value.value;
    const schemaHasConflictingModel = !!inputDocument.definitions.find(hasModelWithNamePredicate(originalName));
    if (schemaHasConflictingModel) {
        throw new graphql_transformer_core_1.InvalidDirectiveError(`Type ${modelName} cannot map to ${originalName} because ${originalName} is a model in the schema.`);
    }
    updateFunction(modelName, originalName);
};
const hasModelWithNamePredicate = (name) => (node) => { var _a; return node.kind === graphql_1.Kind.OBJECT_TYPE_DEFINITION && !!((_a = node.directives) === null || _a === void 0 ? void 0 : _a.find((dir) => dir.name.value === 'model')) && node.name.value === name; };
//# sourceMappingURL=graphql-maps-to-transformer.js.map